#!/bin/bash -x
#SBATCH -J JAFt14
#SBATCH --nodes=4
#SBATCH --ntasks=96
#SBATCH --ntasks-per-node=24
#SBATCH --cpus-per-task=1
#SBATCH -e error-%j.err
#SBATCH -o output-%j.out
#SBATCH --mail-user=flavianojs@live.com
#SBATCH --mail-type=FAIL
##SBATCH -p large
#SBATCH --time=01:59:00


MY_PROG="kkr.x"
NUM_PROCS=${SLURM_NTASKS}
export MKL_NUM_THREADS=1

MY_HOST=`hostname`
MY_DATE=`date`
MY_PATH=$SLURM_SUBMIT_DIR

echo " -----------------------------------------------------------------------------"
echo " ------------------------------ JOB INFORMATION ------------------------------"
echo " -----------------------------------------------------------------------------"
echo " ---- JOBID        :  $SLURM_JOB_ID"
echo " ---- JOBNAME      :  $SLURM_JOB_NAME"
echo " ---- cluster node :  $MY_HOST"
echo " ---- num. of CPUs :  $NUM_PROCS"
echo " - MKL NUM THREADS :  $MKL_NUM_THREADS"
echo " ---- WORKDIR      :  $SLURM_SUBMIT_DIR"
echo " -----------------------------------------------------------------------------"
echo " ----------- start :  $MY_DATE ---------------------------"
echo " -----------------------------------------------------------------------------"
echo ""
echo " PATH TO PROGRAM   : $MY_PATH"
echo " NAME OF EXECUTABLE: $MY_PROG"
echo ""

cd $SLURM_SUBMIT_DIR

ulimit -s unlimited

pref="SYSTEM"
prefoutf="OUT"
work_folder=$pref
index=1
if [ $pref == SYSTEM ] ; then
    work_folder=$pref$index
    outfile=$prefoutf$index
fi
while test -d ./$work_folder ; do
  work_folder=$pref$index
  outfile=$prefoutf$index
  index=`expr $index + 1`
done

mkdir $work_folder
cp     inputcard potential shapefun nonco_angle.dat $work_folder
cd     $work_folder
echo Running directory `pwd`

echo Executing $SLURM_SUBMIT_DIR/$MY_PROG
srun $SLURM_SUBMIT_DIR/$MY_PROG > ../$outfile

rm *unformatted* 2>/dev/null

MY_DATE=`date`
echo ""
echo " -----------------------------------------------------------------------------"
echo " ----------- end   :  $MY_DATE ---------------------------"
echo " -----------------------------------------------------------------------------"



