from math import exp
import numpy as np
import matplotlib.pyplot as plt
from scipy.interpolate import griddata
from numpy.random import uniform, seed

vmax = 2.2

k1 = 0.00000000

# Copied from Spin wave code output
# High symmetry point path location:
#   Path      | Correspondent frequency value
k2 = 0.50000000
k3 = 1.00000000
k4 = 3.94001653
k5 = 4.51736680
k6 = 5.09471707
k7 = 6.01581657
k8 = 6.73351495
k9 = 7.45121333

filename = "outputfiles/disp_unfol_MnSiDisper.dat"
filename_expdataa = "INS_data_Mn5Si3/Q=(Qh,1,0)_10K_dispersion.txt"
filename_expdatab = "INS_data_Mn5Si3/Q=(0,Qk,1)_10K_dispersion.txt"
filename_expdatac = "INS_data_Mn5Si3/Q=(0,3,Ql)_10K_dispersion.txt"

#High resolution for the paper
params = {'font.size': 18,
            'font.family': 'serif',
            'legend.fontsize': 12,
            'mathtext.fontset': 'dejavuserif',
            # 'figure.figsize': (7, 4.5) ,
            'figure.figsize': (7, 12) ,
            'figure.dpi' : 120,
            # 'figure.dpi' : 100,
            'legend.handlelength': 2}

plt.rcParams.update(params)

def read_blocks(input_file, i, j):
    empty_lines = 0
    blocks = [[]]
    for line in open(input_file):
        # Check for empty/commented lines   
        if not line or line.startswith('#') or line == ' \n':
            # If 1st one: new block
            if empty_lines == 0:
                # print('number of line in last block', len(blocks[-1]))
                blocks.append([])
                # print('number of blocks', len(blocks))

            empty_lines += 1
        # Non empty line: add line in current(last) block
        else:
            empty_lines = 0
            # blocks[-1].append(line)
            data = [ float(item) for item in line.split() ]
            blocks[-1].append(data)
            
    print('number of blocks', len(blocks))
    return blocks[i:j + 1]

blocks = read_blocks(filename, 0, -2)
print('Data file read.')
print(np.array(blocks).shape)

expdataa = np.loadtxt(filename_expdataa)
expdatab = np.loadtxt(filename_expdatab)
expdatac = np.loadtxt(filename_expdatac)

x=[]; y=[]; z=[]
for block in blocks:
    # print(np.array(block).shape)
    for line in block:
        x.append(line[0])
        y.append(line[1])
        z.append( sum(line[6:10]) )
        # print(x[-1], y[-1], z[-1])
print('x, y, and z lists created.')

# define grid.
xi = np.linspace(min(x),max(x),900)
yi = np.linspace(min(y),max(y),300)
# grid the data.
zi = griddata((x, y), z, (xi[None,:], yi[:,None]), method='cubic')
print('Gridding finished.')

fig, (ax1, ax2, ax3) = plt.subplots(3, 1)

# CS = plt.contour(xi,yi,zi,15,linewidths=0.5,colors='k')

ax1.contourf(xi,yi,zi,300,cmap=plt.cm.gnuplot, vmax= vmax*1.5)
ax2.contourf(xi,yi,zi,300,cmap=plt.cm.gnuplot, vmax= vmax)
ax3.contourf(xi,yi,zi,300,cmap=plt.cm.gnuplot, vmax= vmax)

# ax1.tricontourf(x,y,z,300,cmap=plt.cm.gnuplot)
# ax2.tricontourf(x,y,z,300,cmap=plt.cm.gnuplot)
# ax3.tricontourf(x,y,z,300,cmap=plt.cm.gnuplot, vmax= vmax)

# plt.colorbar() # draw colorbar

### PANEL a ###
### PANEL b ###
### PANEL c ###

ax1.errorbar(x=(expdataa[:,0]-1.5)*((k3-k1)/1.0+k1), xerr=expdataa[:,1], y=expdataa[:,2], yerr=expdataa[:,3],
            color = 'lime',
            marker = 'o', 
            markersize = 4, 
            linestyle='', 
            linewidth=1,
            capsize=2,
            label = 'exp. neutron data'
            )

ax2.errorbar(x=(expdatab[:,0]-2.)*((k6-k4)/2.)+k4, xerr=expdatab[:,1], y=expdatab[:,2],
            color = 'lime',
            marker = 'o', 
            markersize = 4, 
            linestyle='', 
            linewidth=1,
            capsize=2,
            label = 'exp. neutron data'
            )

ax3.errorbar(x=(expdatac[:,0]-0.5)*((k9-k7)/1.)+k7, xerr=expdatac[:,1], y=expdatac[:,2],
            color = 'lime',
            marker = 'o', 
            markersize = 4, 
            linestyle='', 
            linewidth=1,
            capsize=2,
            label = 'exp. neutron data'
            )

ax1.set_xlim(k1,k2)
ax2.set_xlim(k4,k6)
ax3.set_xlim(k7,k9)

ax1.set_xlabel(r"$\mathbf{Q} = (Q_h, 1, 0)$")
ax1.set_ylabel("Energy (meV)")
ax2.set_xlabel(r"$\mathbf{Q} = (0, Q_k, 1)$")
ax2.set_ylabel("Energy (meV)")
ax3.set_xlabel(r"$\mathbf{Q} = (0, 3, Q_l)$")
ax3.set_ylabel("Energy (meV)")

quart = (k3-k1)/4.
ax1.set_xticks(np.arange(k1,k3+quart,quart))
ax1.set_xticklabels(np.arange(1.5, 2.75, 0.25))
quart = (k6-k4)/4.
ax2.set_xticks( np.arange(k4, k6+quart, quart) )
ax2.set_xticklabels(np.arange(2.0, 4.5, 0.50))
# ax2.set_xlim(k4+quart,k4+3.*quart)
quart = (k9-k7)/4.
ax3.set_xticks( np.arange(k7, k9+quart, quart) )
ax3.set_xticklabels(np.arange(0.5, 1.75, 0.25))

# print(np.arange(k7, k9+quart, quart))
# print(k7, k8, k9)
# print(k7+quart*2.)

# ax1.set_ylim(0,9)

# ax2.tick_params(left = False, right = False , labelleft = False ,
#                 labelbottom = True, bottom = True)

# ax2.set_xticks( [3.75378527, 4.33113554, 4.90848581] )
# ax2.set_xticklabels([2.0, 3.0, 4.0])

# ax3.tick_params(left = False, right = False , labelleft = False ,
                # labelbottom = True, bottom = True)

# plt.legend()
plt.tight_layout()

print('Plotted.')

plt.savefig('dispersion.png')
print('Figure saved.')
# plt.show()

